#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/CGLMacro.h>
#import <OpenGL/gluContext.h>

#import "GLFieldOfViewPatch.h"


@implementation GLFieldOfViewPatch : QCPatch

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 1;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return YES;
}

+ (int)timeModeWithIdentifier:(id)fp8
{
	return 1;
}

- (id)initWithIdentifier:(id)fp8
{
	id z=[super initWithIdentifier:fp8];
	
	if(z)
	{
		// This is the default FOV, as stored in the default QC Model View Matrix
		[inputFieldOfView setDoubleValue: 90.0];
	}

	return z;
}
- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme GL Field of View" forKey:@"name"];
	return at;
}

- (id)setup:(QCOpenGLContext *)context
{

	return context;
}
- (void)cleanup:(QCOpenGLContext *)context
{


}


- (void)enable:(QCOpenGLContext *)context
{

}
- (void)disable:(QCOpenGLContext *)context
{


}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	CGLContextObj cgl_ctx = [context CGLContextObj];
	//int i;
	double fovy = [inputFieldOfView doubleValue];
	double aspect;
	//double aspect = [inputAspectRatio doubleValue];
	if(fovy < 0.5)
		fovy = 0.5;
	else if(fovy > 180.)
		fovy = 180.;
	/*if(aspect < 0.1)	// interpret as 'calculate from context'
	{
		aspect = 4./3.;
	}*/
	
	double theta = fovy * M_PI / 180.0;
	
	double f = 1.0 / tan(theta/2);
	
	// populated according to gluPerspective specs where f (fovy) is used
	//double oldMatrix[16];
	double modifiedMatrix[16];
	//int depth;
	GLint oldMatrixMode;
	glGetIntegerv(GL_MATRIX_MODE,&oldMatrixMode);
	glGetDoublev(GL_MODELVIEW_MATRIX, modifiedMatrix);
	//glGetIntegerv(GL_MAX_PROJECTION_STACK_DEPTH,&depth);

	//NSLog(@"depth: %i",depth);

	glMatrixMode(GL_MODELVIEW);
	// don't push/pop the projection matrix;  it breaks for some reason ( ?? )
	glPushMatrix();
		aspect = modifiedMatrix[5] / modifiedMatrix[0];	// use current aspect ratio.  Derived from f/aspect (om[0]) and f (om[5])
		modifiedMatrix[0] *= f / aspect;
		modifiedMatrix[5] *= f;
		glLoadMatrixd(modifiedMatrix);
		[self executeSubpatches:time arguments:arguments];
	glPopMatrix();
	glMatrixMode(oldMatrixMode);

	return YES;
}

@end
