#import <OpenGL/gl.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/CGLMacro.h>

#import "GLLinePatch.h"


@implementation GLLinePatch : QCPatch

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 1;
}
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}
+ (int)timeModeWithIdentifier:(id)fp8
{
	return 1;
}


- (id)initWithIdentifier:(id)fp8
{
	id z=[super initWithIdentifier:fp8];
	if(z)
	{
		[inputSize setDoubleValue:1.0];
		[inputPattern setIndexValue: 65535];
		[inputPattern setMaxIndexValue: 65535];
		[inputRepeatCount setIndexValue: 1];
		[inputRepeatCount setMaxIndexValue: 256];
		[inputX1 setDoubleValue:-0.5];
		[inputX2 setDoubleValue:0.5];
		[inputColor1 setRed:0.7 green:0.4 blue:0.4 alpha:0.9];
		[inputColor2 setRed:0.4 green:0.4 blue:0.7 alpha:0.9];
		
		[inputDepth setIndexValue:1];	// set normal read/write depth testing
	}
	return z;
}
- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme GL Line" forKey:@"name"];
	return at;
}


- (id)setup:(QCOpenGLContext *)context
{
	return context;
}
- (void)cleanup:(QCOpenGLContext *)context
{
}


- (void)enable:(QCOpenGLContext *)context
{
}
- (void)disable:(QCOpenGLContext *)context
{
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	GLint	blendEnabled;
	GLint	blendSrcFunc;
	GLint	blendDstFunc;
	GLint	smoothLines;
	GLint	oldFactor;
	GLint	oldPattern;
	GLfloat origLineWidth;
	
	float red1,green1,blue1,alpha1;
	float red2,green2,blue2,alpha2;
	
	[inputColor1 getRed:&red1 green:&green1 blue:&blue1 alpha:&alpha1];
	[inputColor2 getRed:&red2 green:&green2 blue:&blue2 alpha:&alpha2];
	
	CGLContextObj cgl_ctx = [context CGLContextObj];
	
	[inputBlending setOnOpenGLContext: context];
	[inputDepth setOnOpenGLContext: context];
	
	glGetIntegerv(GL_LINE_STIPPLE_REPEAT,&oldFactor);
	glGetIntegerv(GL_LINE_STIPPLE_PATTERN,&oldPattern);
	glGetFloatv(GL_LINE_WIDTH, &origLineWidth);
	glGetIntegerv(GL_BLEND_SRC,&blendSrcFunc);
	glGetIntegerv(GL_BLEND_DST,&blendDstFunc);
	blendEnabled = glIsEnabled(GL_BLEND);
	if(!blendEnabled)
		glEnable(GL_BLEND);
	smoothLines = glIsEnabled(GL_LINE_SMOOTH);
	if(!smoothLines)
		glEnable(GL_LINE_SMOOTH);
	glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
	
	glEnable(GL_LINE_STIPPLE);
	glLineStipple([inputRepeatCount indexValue],[inputPattern indexValue]);
	glLineWidth([inputSize doubleValue]);
	
	glBegin(GL_LINES);
		glColor4f(red1,green1,blue1,alpha1);
		glVertex3f([inputX1 doubleValue],[inputY1 doubleValue],[inputZ1 doubleValue]);
		glColor4f(red2,green2,blue2,alpha2);
		glVertex3f([inputX2 doubleValue],[inputY2 doubleValue],[inputZ2 doubleValue]);
	glEnd();
	
	glBlendFunc(blendSrcFunc, blendDstFunc);
	if(!blendEnabled)
		glDisable(GL_BLEND);
	if(!smoothLines)
		glDisable(GL_LINE_SMOOTH);
	glLineWidth(origLineWidth);
	glLineStipple(oldFactor, oldPattern);

	[inputDepth unsetOnOpenGLContext: context];
	[inputBlending unsetOnOpenGLContext: context];

	return YES;
}

@end
