#import "GLReadPixels.h"
#import "QCImage.h"
#import <OpenGL/gl.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/CGLMacro.h>

@implementation GLReadPixels : QCPatch

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 0;
}
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}
+ (int)timeModeWithIdentifier:(id)fp8
{
	return 1;
}

- (id)initWithIdentifier:(id)fp8
{
	id z=[super initWithIdentifier:fp8];

	if(z)
	{
		width = 0;
		height = 0;
		cs = CGColorSpaceCreateDeviceRGB();
	}

	return z;
}

- (void)dealloc
{
	free(data);
	free(flipData);

	CGColorSpaceRelease(cs);

	[super dealloc];
}

- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme GL Read Pixels" forKey:@"name"];
	return at;
}

- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	CGLContextObj cgl_ctx = [context CGLContextObj];
	
	unsigned int i, lineBytes;

	GLint viewPort[4];
	
	glGetIntegerv(GL_VIEWPORT, viewPort);
	
	if(width != viewPort[2] || height != viewPort[3])
	{
		if(data)
			free(data);
		if(flipData)
			free(flipData);
		data = (char*)malloc(sizeof(char)* 4 *viewPort[2] * viewPort[3]);
		flipData = (char*)malloc(sizeof(char)* 4 *viewPort[2] * viewPort[3]);
	}
	
#ifdef __BIG_ENDIAN__
	glReadPixels(0,0,viewPort[2],viewPort[3], GL_RGBA, GL_UNSIGNED_INT_8_8_8_8, data);
#else
	glReadPixels(0,0,viewPort[2],viewPort[3], GL_BGRA, GL_UNSIGNED_INT_8_8_8_8, data);
#endif
//	glReadPixels(0,0,viewPort[2],viewPort[3], GL_DEPTH_COMPONENT, GL_FLOAT, data);

	lineBytes = viewPort[2] * 4;
	for(i=0;i<viewPort[3];++i)
		bcopy(data+i*lineBytes,flipData+(viewPort[3]-i-1)*lineBytes,lineBytes);
	
	CGContextRef  bitmapContext = CGBitmapContextCreate(
									flipData, 
									viewPort[2], 
									viewPort[3], 
									8,
									lineBytes,
									cs,
#ifdef __BIG_ENDIAN__
									kCGImageAlphaPremultipliedLast);
#else
									kCGImageAlphaPremultipliedFirst);
#endif
	CGImageRef    bitmapImage = CGBitmapContextCreateImage(bitmapContext);
	QCImage *qcglOut=[[QCImage alloc] initWithCGImage:bitmapImage options:0];
	
	CGImageRelease(bitmapImage);
	CGContextRelease(bitmapContext);
	
	[outputImage setImageValue:qcglOut];
	[qcglOut release];
	//free(flipData);
	//free(data);
	
	return YES;
}

@end
