#import <OpenGL/gl.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/CGLMacro.h>

#import "GLPointPatch.h"

@implementation GLPointPatch : QCPatch

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 1;
}
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}
+ (int)timeModeWithIdentifier:(id)fp8
{
	return 1;
}


- (id)initWithIdentifier:(id)fp8
{
	id z=[super initWithIdentifier:fp8];
	if(z)
	{
		[inputSize setDoubleValue:8.0];
		[inputColor setRed:0.7 green:0.4 blue:0.4 alpha:0.9];
		
		[inputDepth setIndexValue:1];	// set normal read/write depth testing
	}
	return z;
}
- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme GL Point" forKey:@"name"];
	return at;
}


- (id)setup:(QCOpenGLContext *)context
{
	//imageManager = [QCImageManager sharedSoftwareImageManager];
	return context;
}
- (void)cleanup:(QCOpenGLContext *)context
{
}


- (void)enable:(QCOpenGLContext *)context
{
}
- (void)disable:(QCOpenGLContext *)context
{
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	GLboolean	blendEnabled;
	GLboolean	smoothPoints;
	GLint	blendSrcFunc;
	GLint	blendDstFunc;
	GLfloat origPointSize;
	GLfloat oldAttenuation[3];
	
	float red,green,blue,alpha;
	
	if([inputSize doubleValue] <= 0.0)
		return YES;	// invisible -- do nothing
	
	[inputColor getRed:&red green:&green blue:&blue alpha:&alpha];
	
	/* CGLMacros use 'cgl_ctx' */
	CGLContextObj cgl_ctx = [context CGLContextObj];
	
	glGetFloatv(GL_POINT_SIZE,&origPointSize);

	[inputBlending set: cgl_ctx];
	[inputDepth set: cgl_ctx];

	//id image = [QCPlugInInputImage initWithImage:[inputImage imageValue] context:cgl_ctx];
	//[image bindTextureRepresentationToCGLContext:cgl_ctx textureUnit:GL_TEXTURE0 normalizeCoordinates:YES];

	//id image = [[inputImage imageValue] createCroppedImageWithRect:[[inputImage imageValue] bounds]];
	//id f = [[inputImage imageValue] createCIImageForManager:imageManager withOptions:nil];
	//NSLog(@"f (%@) %@",[f className], f);
	//[inputImage setImageValue:f];
	
	//NSLog(@" (%f,%f), %fx%f\n", [image bounds].origin.x, [image bounds].origin.y, [image bounds].size.width, [image bounds].size.height);

	if([inputImage imageValue] != nil)
	{
		[inputImage set:cgl_ctx unit: GL_TEXTURE0 useTransformationMatrix:NO];
		/*[inputImage set: cgl_ctx
				unit: GL_TEXTURE0
				fromBounds: [[inputImage imageValue] bounds]//NSMakeRect(0,0,500,150)//[[inputImage imageValue] bounds]
				withTarget: GL_TEXTURE_2D
				mipmappingLevels: 0
				matrix: NULL];*/
		glEnable(GL_POINT_SPRITE);
		glTexEnvf(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);
	}

	// This version doesn't handle non-alpha'd input correctly ... no idea why.
	/*			unit: GL_TEXTURE0 
				withBounds: [[inputImage imageValue] bounds]
				transformation: nil
				target: GL_TEXTURE_2D
				mipmappingLevels: 0
				matrix: NULL];*/
	
	glGetIntegerv(GL_BLEND_SRC,&blendSrcFunc);
	glGetIntegerv(GL_BLEND_DST,&blendDstFunc);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	blendEnabled = glIsEnabled(GL_BLEND);
	if(!blendEnabled)
		glEnable(GL_BLEND);
	smoothPoints = glIsEnabled(GL_POINT_SMOOTH);
	if(smoothPoints == FALSE)
		glEnable(GL_POINT_SMOOTH);
		
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);

	if([inputAttenuate booleanValue])
	{
		float attenuate[3] = { 1.0, 1.0, 1.0 };	/* this may not be correct attenuation, but it looks pretty close */
		glGetFloatv(GL_POINT_DISTANCE_ATTENUATION, oldAttenuation);
		glPointParameterfv(GL_POINT_DISTANCE_ATTENUATION,attenuate);
		glPointParameterf (GL_POINT_SIZE_MAX, 128.0);
		glPointParameterf (GL_POINT_SIZE_MIN, 1.0);
	}
	
	glPointSize([inputSize doubleValue]);
	
	glBegin(GL_POINTS);
		glColor4f(red,green,blue,alpha);
		glVertex3f([inputX doubleValue],[inputY doubleValue],[inputZ doubleValue]);
	glEnd();

	glPointSize(origPointSize);
	glBlendFunc(blendSrcFunc, blendDstFunc);
	if(!blendEnabled)
		glDisable(GL_BLEND);
	if(!smoothPoints)
		glDisable(GL_POINT_SMOOTH);
	glDisable(GL_POINT_SPRITE);

	//[image unbindTextureRepresentationFromCGLContext:cgl_ctx textureUnit:GL_TEXTURE0];

	if([inputAttenuate booleanValue])
	{
		glPointParameterfv(GL_POINT_DISTANCE_ATTENUATION,oldAttenuation);
	}
	if([inputImage imageValue] != nil)
		[inputImage unset: cgl_ctx unit:GL_TEXTURE0];
	[inputDepth unset: cgl_ctx];
	[inputBlending unset: cgl_ctx];

	return YES;
}

@end
