#import "QCPatch.h"
#import "QCOpenGLContext.h"

#import "QCBooleanPort.h"

@interface QCProgrammablePatch : QCPatch
{
    NSMutableDictionary *_sources;
    NSMutableDictionary *_compileStatus;
    GFList *_parameterTypes;
    NSMutableArray *_parameterPorts;
    GFList *_resultTypes;
    NSMutableArray *_resultPorts;
    void *_unused4[4];
}

+ (id)allocWithZone:(struct _NSZone *)fp8;
+ (Class)portClassFromParameterInfo:(id)fp8;
+ (int)indicesType;
+ (id)sourceTypes;
+ (id)stateKeysWithIdentifier:(id)fp8;
+ (id)serializedStateKeysWithIdentifier:(id)fp8;
- (BOOL)isSourceType:(id)fp8;
- (id)initWithIdentifier:(id)fp8;
- (void)dealloc;
- (id)valueForUndefinedKey:(id)fp8;
- (void)setValue:(id)fp8 forUndefinedKey:(id)fp12;
- (void)recompileSourceOfType:(id)fp8;
- (void)setSource:(id)fp8 ofType:(id)fp12;
- (id)sourceOfType:(id)fp8;
- (id)compileStatusForSourceOfType:(id)fp8;
- (id)compileSourceOfType:(id)fp8;
- (id)attributesForParameterPortWithInfo:(id)fp8 name:(id)fp12 index:(unsigned int)fp16;
- (id)attributesForResultPortWithInfo:(id)fp8 name:(id)fp12 index:(unsigned int)fp16;
- (BOOL)_updateTypes:(id)fp8 ports:(id)fp12 list:(id)fp16 direction:(int)fp20;
- (BOOL)setParameterList:(id)fp8;
- (id)parameterList;
- (id)parameterPorts;
- (BOOL)setResultList:(id)fp8;
- (id)resultList;
- (id)resultPorts;
- (id)state;
- (BOOL)setState:(id)fp8;

@end

@interface QCExpression : QCProgrammablePatch
{
    QCNumberPort *outputResult;
    NSObject *_evaluator;
}

+ (BOOL)isSafe;
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8;
+ (id)sourceTypes;
- (void)dealloc;
- (void)setSource:(id)fp8 ofType:(id)fp12;
- (id)compileSourceOfType:(id)fp8;
- (BOOL)execute:(id)fp8 time:(double)fp12 arguments:(id)fp20;

@end

@interface QCJavaScript : QCProgrammablePatch
{
    GFList *_inputList;
    GFList *_outputList;
    BOOL _testDrive;
    BOOL _newMode;
    void *_jsContext;
    void *_global;
    void *_function;
}

+ (id)sourceTypes;      // IMP=0x00009c13
+ (BOOL)isSafe; // IMP=0x00009c48
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8; // IMP=0x00009c52
+ (id)stateKeysWithIdentifier:(id)fp8;  // IMP=0x00009fed
+ (unsigned int)stateVersionWithIdentifier:(id)fp8;     // IMP=0x0000a083
- (id)_parseMain:(id)fp8 strippedSource:(id *)fp12;     // IMP=0x0000a435
- (BOOL)setup:(id)fp8;  // IMP=0x00008011
- (const struct OpaqueJSValue **)_createEmptyInputArgumentArray;        // IMP=0x0000cd28
- (id)_checkMainFunctionOutputArguments:(struct OpaqueJSValue *)fp8;    // IMP=0x00008082
- (id)compileSourceOfType:(id)fp8;      // IMP=0x00008b27
- (unsigned int)numberOfInputs; // IMP=0x000095d1
- (void)setNumberOfInputs:(unsigned int)fp8;    // IMP=0x000095f5
- (unsigned int)numberOfOutputs;        // IMP=0x000098f2
- (void)setNumberOfOutputs:(unsigned int)fp8;   // IMP=0x00009916
- (id)initWithIdentifier:(id)fp8;       // IMP=0x00009c59
- (void)_destroyJavaScript;     // IMP=0x00009ed3
- (void)finalize;       // IMP=0x00009f0a
- (void)dealloc;        // IMP=0x00009f5a
- (void)cleanup:(id)fp8;        // IMP=0x00009fd5
- (id)state;    // IMP=0x0000a08d
- (BOOL)setState:(id)fp8;       // IMP=0x0000a1f8
- (BOOL)execute:(id)fp8 time:(double)fp12 arguments:(id)fp20;   // IMP=0x0000b622

@end

@interface QCJavaScript (Kineme)
+ (int)executionModeWithIdentifier:(id)fp8;
@end

@interface GLScriptPatch : QCJavaScript
{

}
@end